-- DEMONSTRATION OF THE INTERPRETER IN THE PROGRAM "DEMAGOG"
-- SIMPLE AND FAST AUDIO-CONVERTER
-- Author: balaamster
-- v. 1.0, 2018

-- In the home folder 'Demagog' should be utilities ffmpeg.exe and mediainfo.exe and 'fon' folder.
-- Copy mp3-files with background sound to the 'fon' folder. Backgrounds are selected automatically, randomly.

-----------------------------------------------------------------------------
if CurrentLang() == 'Russian' then
  msg = "\13   'Demagog'    ffmpeg.exe  mediainfo.exe   'fon'.\13 mp3-      'fon'.   ,  . ?"
  src_path_cap = '    '
  src_fsel_cap = '   '
  dst_fsel_cap = '    '
  dst_path_cap = '     '
  dst_btrt_cap = ' '
  dst_echo_cap = '   ?'
  dst_fon_cap = '     ?'
  dst_tmp_cap = '   '
  fon_vol_cap = '   '
  msg_notfound = '     '
  msg_fin = ' '
  msg_yes = ''
  msg_no = ''
  msg_succ = ':'
  msg_err = ':'
  msg_diff = ''
  msg_sec = '.'
  info_str = [[
 : %s
 : %s
 : %s
 : %s
 : %s
]]
else
  msg = "AUDIO-CONVERTER\13In the home folder 'Demagog' should be utilities ffmpeg.exe and mediainfo.exe and 'fon' folder.\13Copy mp3-files with background sound to the 'fon' folder. Backgrounds are selected automatically, randomly. Continue?"
  src_path_cap = 'Select the source directory'
  src_fsel_cap = 'Select source file format'
  dst_fsel_cap = 'Select the target format to convert'
  dst_path_cap = 'Select target directory for converted files'
  dst_btrt_cap = ' '
  dst_echo_cap = 'Add reverb to speech?'
  dst_fon_cap = 'Add background sound to speech files?'
  dst_tmp_cap = 'Choose your preferred speech speed'
  fon_vol_cap = 'Select background sound volume'
  msg_notfound = 'No files found with background sound'
  msg_fin = 'Processing completed'
  msg_yes = 'Yes'
  msg_no = 'No'
  msg_succ = 'SUCCESS:'
  msg_err = 'ERROR:'
  msg_diff = 'difference'
  msg_sec = 'sec.'
  info_str = [[
Processed directory: %s
Target directory: %s
Target format: %s
Target bitrate: %s
Speech speed: %s
]]
end
-----------------------------------------------------------------------------

if not MessageDialog(msg) then goto HALT end

local aformats = {'mp3','wav','ogg'}
local br_max_table = {mp3=128, ogg=80}
local tempos = {}
WNew(0)
for i = 0.5,2,0.1 do
    table.insert(tempos, i)
end
local fon_weights = {}
for i = 0.1,1,0.1 do
    table.insert(fon_weights, i)
end

local function get_files(path)
    local sel_type = Menu(src_fsel_cap, aformats, 2)
    if sel_type == 0 then return nil end
    local fnames = AllFiles(path, '*.'..aformats[sel_type])
    return fnames
end

local function rnd_fon(fdir, ftype)
    local fon_home = HomeFolder(fdir)
    local fon_files = AllFiles(fon_home,'*.'..ftype)
    if #fon_files == 0 then return nil end
    return fon_home..fon_files[math.random(1,#fon_files)]
end

local function get_duration(path)
    info = HomeFolder()..'MediaInfo.exe'
    local cmd = string.format('CHCP 1251 | "%s" --Inform=Audio;%%Duration%% "%s"', info, path)
    for i in io.popen(cmd):lines() do
      if string.match(i,'^%d+') ~= nil then
          duration = tonumber(i)
          return math.floor(duration/1000)
      end
    end
    return nil
end

local function check2audio(path_a, path_b)
    local a = get_duration(path_a)
    local b = get_duration(path_b)
   local diff = math.abs(a - b) 
    return (diff <= 1) or false, a, b, diff
end

local src_path = Folders(src_path_cap)
if src_path == false then goto HALT end

local files = get_files(src_path)
if not files then goto HALT end

local prefix = ''
local dst_path = Folders(dst_path_cap, src_path, true)
if dst_path == false then goto HALT end
if dst_path == src_path then prefix = '_temp_' end

local sel_type = Menu(dst_fsel_cap, aformats)
if sel_type == 0 then goto HALT end
local dst_format = aformats[sel_type]

local bitrates = {}
local bitrate
local br
if dst_format ~= 'wav' then
    local br_max = br_max_table[dst_format]
    for i = 32,br_max,16 do
        table.insert(bitrates,tostring(i))
    end
    local sel_br = Menu(dst_btrt_cap, bitrates)
    if sel_br == 0 then goto HALT end
    br = bitrates[sel_br]
    bitrate = '-b:a '..br..'k'
end

local add_echo = Menu(dst_echo_cap, {msg_yes,msg_no})
if add_echo == 0 then goto HALT end

local encoder = HomeFolder()..'ffmpeg.exe'

local add_fon
local fon_weight
local enc_opt
local echo_flt = ''
local sel_fon = Menu(dst_fon_cap, {msg_yes,msg_no})
if sel_fon == 0 then
    goto HALT
elseif sel_fon == 1 then
   if add_echo == 1 then echo_flt = '[b]aecho=0.8:0.6:20^|40:0.2^|0.2[b];' end
   local sel_fon_weight = Menu(fon_vol_cap, fon_weights)
   if sel_fon_weight == 0 then goto HALT end
   fon_weight = fon_weights[sel_fon_weight]
   add_fon = true
   enc_opt = '"%s" -loglevel -8 -y -i "%s" -filter_complex "amovie=fon.mp3:loop=10[s];[0]atempo=%s[b];%s[s]afade=t=in:ss=0:d=15[s];[s]afade=t=out:st=%s:d=15[s];[b][s]amix=inputs=2:duration=first:weights=1 %s" -ac 1 %s "%s"'
elseif sel_fon == 2 then
   if add_echo == 1 then echo_flt = ',aecho=0.8:0.6:20^|40:0.2^|0.2' end
   add_fon = false
   enc_opt = '"%s" -loglevel -8 -y -i "%s" -filter:a "atempo=%s%s" -ac 1 %s "%s"'
end

local sel_tempo = Menu(dst_tmp_cap, tempos, 6)
if sel_fon == 0 then goto HALT end
local tempo = tempos[sel_tempo]

local ans = MessageDialog(string.format(info_str,src_path,dst_path, dst_format, br, tempo))
if not ans then goto HALT end

local tmp_bat = string.format('%s\\%s', os.getenv("TEMP"),"tmp.bat")
for i = 1, #files do
    local in_file = src_path..files[i]
    local out_file = dst_path..prefix..SplitFileName(in_file)[2]..'.'..dst_format
    if add_fon then
        local fon_path = rnd_fon('fon', 'mp3')
        if not fon_path then
            ShowMessage(msg_notfound)
            goto HALT
        end
        local fon_cmd = string.format('CHCP 1251 | COPY /Y "%s" "%s"', fon_path, HomeFolder()..'fon.mp3')
        os.execute(fon_cmd)
        local fade_out_time = math.floor(get_duration(in_file) / tempo - 15)
        cmd = string.format(enc_opt, encoder, in_file, tempo, echo_flt, fade_out_time, fon_weight, bitrate, out_file)
    else
        cmd = string.format(enc_opt, encoder, in_file, tempo, echo_flt, bitrate, out_file)
    end
    SaveToFile({cmd}, tmp_bat)
    if FileExists(tmp_bat) then
        os.execute("chcp 1251 | "..tmp_bat)
        os.remove(tmp_bat)
    end
--[[  local res, a, b, diff = check2audio(in_file, out_file)
  if res then res = msg_succ else res = msg_err end
  local log_str = string.format('%s %s '..msg_diff..': %s '..msg_sec, res, out_file, diff)
  WLog(log_str)
]]--
end

Sound()
ShowMessage(msg_fin)

::HALT::
